#include <stdio.h>
#include <stdlib.h>

struct list {
    int val;
    struct list *next;
};

/* Dichiarazione di alcune delle funzioni viste a lezione; le funzioni
   sono definite sotto (non devono essere implementate), e possono
   essere usate se serve. */
struct list *list_create(int v, struct list *t);
int list_length(struct list *L);
void list_destroy(struct list *L);
int is_empty(struct list *L);
int list_equal(struct list *L1, struct list *L2);
void list_print(struct list *L);

/* FUNZIONE DA REALIZZARE
   ======================

   Realizzare la funzione f(L1, L2) in base alla specifica
   seguente. La funzione accetta come parametro due liste L1, L2 di
   interi, non necessariamente della stessa lunghezza. La funzione
   deve restituire una NUOVA lista L, avente lunghezza uguale al
   massimo della lunghezza di L1 ed L2. Gli elementi di L sono il
   _massimo_ tra gli elementi corrispondenti di L1 e L2; se una delle
   due liste e' piu' corta, si prosegue con gli elementi dell'altra.

   Ad esempio, se 

   L1 = (1, 10, 9, -5, 12)
   L2 = (13, 1, -3, 5, 12, 31, 45)

   allora la funzione deve costruire la nuova lista

   L = (13, 10, 9, 5, 12, 31, 45)

   (il primo elemento di L e' il massimo dei primi elementi di L1 e
   L2; il secondo elemento di L e' il massimo dei secondi elementi di
   L1 e L2; dato che L1 e' piu' corta, si completa il risultato con
   gli elementi di L2).

   Come caso particolare, se L1, L2 sono entrambe la lista vuota, la
   funzione restituisce la lista vuota.

   La funzione non deve modificare L1 ed L2, ma restituire una nuova
   lista.

   E' possibile definire ulteriori funzioni di supporto, se
   necessario.

   Suggerimento: in caso di difficolta' si consiglia di iniziare
   assumendo che L1 ed L2 abbiano la stessa lunghezza; se si scrive
   una funzione corretta solo in questo caso si ricevera' comunque un
   punteggio parziale (circa 70% del punteggio massimo).
*/
struct list *f(struct list *L1, struct list *L2)
{
    if (is_empty(L1) && is_empty(L2)) {
        return NULL;
    } else {
        if (is_empty(L1)) {
            return list_create(L2->val, f(L1, L2->next));            
        } else {
            if (is_empty(L2)) {
                return list_create(L1->val, f(L1->next, L2));
            } else {
                return list_create(L1->val > L2->val ? L1->val : L2->val,
                                   f(L1->next, L2->next));
            }
        }
    }
}

/**
 ** Le funzioni seguenti sono gia' implementate 
 **/

/* Crea (mediante malloc() ) e restituisce un puntatore ad un nuovo
   nodo di una lista; il nodo contiene il valore v e punta a t come
   elemento successivo. Il chiamante e' responsabile per deallocare
   mediante free() o simili la zona di memoria restituita da questa
   funzione, quando non piu' utilizzata */
struct list  *list_create(int v, struct list *t)
{
    struct list *r=(struct list *)malloc(sizeof(struct list));
    r->val = v;
    r->next = t;
    return r;
}

/* Restituisce la lunghezza (numero di nodi) della lista L; se L e' la
   lista vuota, restituisce 0 */
int list_length(struct list *L)
{
    /* Versione ricorsiva */
    if ( NULL == L ) {
        return 0;
    } else {
        return (1 + list_length(L->next));
    }
}

/* Libera la memoria occupata da tutti i nodi della lista L */
void list_destroy(struct list *L)
{
    /* versione ricorsiva */
    if ( L != NULL ) {
        list_destroy(L->next);
        L->next = NULL; /* non necessario... */
        free(L);
    }
}

/* Restituisce 1 se e solo se L e' la lista vuota, 0 altrimenti */
int is_empty(struct list *L)
{
    return (NULL == L);
}

/* Stampa il contenuto della lista L */
void list_print(struct list *L)
{
    printf("(");
    while (L != NULL) {
        printf("%d", L->val);
        if (L->next != NULL) printf(", ");
        L = L->next;
    }
    printf(")");
}

int list_equal(struct list *L1, struct list *L2)
{
    if (is_empty(L1) || is_empty(L2)) {
        /* Se siamo qui significa che almeno una tra L1 e L2 e' vuota
           (possono esserlo entrambe). Allora le liste sono uguali se
           e solo se sono ENTRAMBE vuote */
        return (is_empty(L1) && is_empty(L2));
    } else {
        if (L1->val != L2->val) {
            return 0;
        } else {
            return list_equal(L1->next, L2->next);
        }
    }
}

struct list *list_from_array(int v[], int n)
{
    if ( n == 0 ) {
        return NULL;
    } else {
        return list_create( v[0], list_from_array(v+1, n-1));
    }
}

void test(struct list *L1, struct list *L2, struct list *expect)
{
    struct list *result = f(L1, L2);

    if (list_equal(result, expect)) {
        printf("Test OK\n");
    } else {
        printf("Test FALLITO: L1=");
        list_print(L1);
        printf(", L2=");
        list_print(L2);
        printf(", risultato atteso=");
        list_print(expect);
        printf(", risultato ottenuto=");
        list_print(result);
        printf("\n");
    }
    /* list_destroy(result); */
}

#define LIST_FROM_ARRAY(a) list_from_array(a, (sizeof(a)/sizeof(a[0])))

int main( void )
{
    int A1[] = {1, 10, 9, -5, 12};
    int B1[] = {13, 1, -3, 5, 12};
    int out1[] = {13, 10, 9, 5, 12};
    /* A lezione non abbiamo visto l'uso della macro in linguaggio C;
       e' sufficiente sapere che l'espressione

       LIST_FROM_ARRAY(A1)

       equivale a scrivere

       list_from_array(A1, (sizeof(A1)/sizeof(A1[0])))
    */
    struct list *A1_in = LIST_FROM_ARRAY(A1);
    struct list *B1_in = LIST_FROM_ARRAY(B1);
    struct list *L1_out = LIST_FROM_ARRAY(out1);
    
    int A2[] = {1, 2, 3, 4, 5};
    int B2[] = {1, -10, 30, 20, -2};
    int out2[] = {1, 2, 30, 20, 5};
    struct list *A2_in = LIST_FROM_ARRAY(A2);
    struct list *B2_in = LIST_FROM_ARRAY(B2);
    struct list *L2_out = LIST_FROM_ARRAY(out2);

    int A3[] = {1};
    int B3[] = {-10};
    int out3[] = {1};
    struct list *A3_in = LIST_FROM_ARRAY(A3);
    struct list *B3_in = LIST_FROM_ARRAY(B3);
    struct list *L3_out = LIST_FROM_ARRAY(out3);

    struct list *A4_in = NULL;
    struct list *B4_in = NULL;
    struct list *L4_out = NULL;

    int A5[] = {3, 12, 31, 28, 12};
    int B5[] = {13, 12};
    int out5[] = {13, 12, 31, 28, 12};
    struct list *A5_in = LIST_FROM_ARRAY(A5);
    struct list *B5_in = LIST_FROM_ARRAY(B5);
    struct list *L5_out = LIST_FROM_ARRAY(out5);

    int A6[] = {3};
    int B6[] = {-11, -10, -9, -8, -7};
    int out6[] = {3, -10, -9, -8, -7};
    struct list *A6_in = LIST_FROM_ARRAY(A6);
    struct list *B6_in = LIST_FROM_ARRAY(B6);
    struct list *L6_out = LIST_FROM_ARRAY(out6);

    int B7[] = {-11, -10, -9, -8, -7};
    int out7[] = {-11, -10, -9, -8, -7};
    struct list *A7_in = NULL;
    struct list *B7_in = LIST_FROM_ARRAY(B7);
    struct list *L7_out = LIST_FROM_ARRAY(out7);

    printf("=== Inizio test ===\n");
    test(A1_in, B1_in, L1_out);
    test(A2_in, B2_in, L2_out);
    test(A3_in, B3_in, L3_out);
    test(A4_in, B4_in, L4_out);
    test(A5_in, B5_in, L5_out);
    test(A6_in, B6_in, L6_out);
    test(A7_in, B7_in, L7_out);
    printf("=== Fine test ===\n");
    
    return 0;
}
