#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* FUNZIONE DA REALIZZARE
   ======================

   Realizzare la funzione f(s) in base alla specifica seguente. La
   funzione accetta come parametro una stringa di lunghezza n>=1, i
   cui caratteri possono essere '0' oppure '1'. La funzione
   restituisce una NUOVA stringa r avente lunghezza (n+1). I primi n
   caratteri di r devono essere uguali a quelli di s; l'ultimo
   carattere a destra di r vale '0' oppure '1' in modo tale che in r
   ci sia un numero _DISPARI_ di caratteri '1'.

   Ad esempio, se s = "1010", allora r = "10101"; se s = "1101",
   allora r = "11010". Si noti come r si ottenga appendendo
   ad s il carattere '0' oppure '1', in modo che cio' che si ottiene
   contenga un numero dispari di '1'.

   Il numero di test forniti in questa funzione e' insufficiente per
   una verifica approfondita del risultato. Si aggiungano almeno TRE
   ulteriori casi di test al main. Si ricordi comunque che una
   funzione che supera i test non e' necessariamente corretta.

   E' sempre possibile definire ulteriori funzioni di supporto, se
   necessario.
*/
char *f( char *s )
{
    int n = strlen(s);
    /* Attenzione: dobbiamo allocare (n+2) byte di memoria:
       - nei primi n memorizziamo gli n caratteri di s;
       - nel carattere successivo memorizziamo '0' oppure '1'
       - nel carattere successivo memorizziamo il terminatore

       Nota: non andava bene scrivere

       char result[n+2];

       In quest'ultimo caso, infatti, il compilatore segnala un warning:

       function returns address of local variable

       Infatti, come detto a lezione, gli array dichiarati in questo
       modo (cioe' specificando la lunghezza tra parentesi quadre)
       vengono deallocati quando si esce dal blocco in cui sono
       definiti, esattamente come le variabili normali. Quindi l'array
       result dichiarato come sopra non esisterebbe al di fuori di
       questa funzione. Invece, la memoria riservata con malloc()
       continua ad essere riservata fino a quando non viene
       esplicitamente liberata con la free().
    */
    char *result = (char*)malloc(n+2);
    int i, c=0;
    for (i=0; i<n; i++) {
        result[i] = s[i];
        if (s[i] == '1') c++;
    }
    if (c%2 != 0) {
        result[i] = '0';
    } else {
        result[i] = '1';
    }
    result[i+1] = '\0';
    return result;
}

void test(char *s, char *atteso)
{
    char *ottenuto = f(s);
    if ((ottenuto != NULL) && (strcmp(atteso, ottenuto) == 0)) {
        printf("Test OK\n");
    } else {
        printf("Test FALLITO: s=%s, risultato atteso=%s, risultato ottenuto=%s\n", s, atteso, ottenuto);
    }
    free(ottenuto);
}

int main( void )
{
    test("1010", "10101");
    test("1101", "11010");
    test("0", "01");
    test("1", "10");
    test("00", "001");
    test("01", "010");
    test("10", "100");
    test("11", "111");
    test("0000", "00001");
    test("0001", "00010");
    test("11111111", "111111111");
    test("11110111", "111101110");
    return 0;
}
